var $ = jQuery;

$(function () {
    // Секция инициализации
    var optionsViewData = {
        'absence': {
            sendingOption: 'absence',
            messageOK: 'Неявка выставлена',
            messageFail: 'Установка неявки не удалась'
        },
        'absence_unset': {
            sendingOption: null,
            messageOK: 'Неявка отменена',
            messageFail: 'Отмена неявки не удалась'
        },
        'pass': {
            sendingOption: 'pass',
            messageOK: 'Автомат выставлен',
            messageFail: 'Установка автомата не удалась'
        },
        'pass_unset': {
            sendingOption: null,
            messageOK: 'Автомат отменен',
            messageFail: 'Отмена автомата не удалась'
        }
    };

    var json_settings = (function () {
        var $hiddenInfo = $('#hidden_div');
        var data = $.parseJSON($hiddenInfo.html());
        $hiddenInfo.remove();
        return data;
    })();
    var pageType = $('#pageType').val();
    var disciplineType = $('#ExamType').val();
    var cancelFlag = false;
    var g_col;
    var g_row;
    var g_isFocusCell = false; // Стоит фокус на ячейке или нет
    var g_oldRateCell = null;
    var g_submoduleID = null;
    var g_studentID = null;
    var g_submoduleTitle = '';
    var g_submoduleMaxRate = 0;
    var g_stdName = '';

    var jCommonCell = $('.commonCell').not('.absenceCell').not('.autoPass');
    var jTdInfo_wrap = $('#tdInfo_wrap');
    var jTdInfo = jTdInfo_wrap.children('#tdInfo');
    var g_disciplineID = $('#disciplineID').val();

    var StudentState = {
        successful: 0, // успевающий
        beforeExam: 1, // экзамен еще не состоялся
        notPassed: 2, // пытался сдать экзамен и не сдал
        notAdmitted: 3, // недопущенный задолженник
        admitted: 4, // допуск получен, экзамен еще не состоялся
        admittedAndPassed: 5, // задолженность закрыта полностью
        admittedAndNotPassed: 6 // допуск получен, экзамен не сдал
    };

    // on page loaded
    // Настройки дисциплины:
    // + ID - id дисциплины
    // + studyGroupID_Filter - studyGroupID для фильтра (Эффект памяти)

    controlVisualization();
    $('div.main_content').ready(AdjustTable);
    $(window).resize(AdjustTable);

    // выставить фильтр по группам
    (function () {
        var groupID = json_settings['GroupID_Filter'];
        if (groupID) {
            filterGroups(groupID);
            $(".groupSelector [value='" + groupID + "']").attr('selected', 'selected');
        }
    })();

    function AdjustTable() {
        var jWrap = $('div.main_layer');
        var jTable = $('div.main_content');
        var tableWidth = jTable.get(0).scrollWidth;
        // check scroll bar
        if (tableWidth <= jTable.width() + 10) {
            return;
        }

        //correct
        tableWidth *= 1.1;
        var maxWidth = $(window).width() * 0.95;
        var newWidth = (tableWidth > maxWidth) ? maxWidth : tableWidth;
        jWrap.css('max-width', newWidth);
    }

    function deriveStudentState(semesterRate, extraRate, examRate, autopass, firstAttempt) {
        var admissionRate = semesterRate + extraRate;

        if (admissionRate >= 38) {
            if ((semesterRate >= 38) && firstAttempt) {
                if ((examRate >= 22) || autopass)
                    return StudentState.successful; // успевающий
                else if (examRate == -1)
                    return StudentState.beforeExam; // экзамен еще не состоялся
                else
                    return StudentState.notPassed; // пытался сдать экзамен и не сдал
            } else {
                if (examRate == -1)
                    return StudentState.admitted; // допуск получен, экзамен еще не состоялся
                else if (examRate >= 22)
                    return StudentState.admittedAndPassed; // задолженность закрыта
                else
                    return StudentState.admittedAndNotPassed; // допуск получен, экзамен не сдал
            }
        } else
            return StudentState.notAdmitted; // недопущенный задолженник
    }

    function setCellAccess(cell, access) {
        if (access)
            $(cell).removeAttr('disabled');
        else
            $(cell).attr('disabled', true);
    }

    function controlRowVisualization(jRow) {

        if ((pageType !== 'exam') || (disciplineType !== 'exam')) { // || (json_settings.ExamType !== 'exam')) {
            return;
        }

        var jAutoPassCheckBox = jRow.children('.autoPass').children('.autoPassCheck');
        var jAbsenceCheckBoxes = jRow.children('.absenceCell').children('.absenceCheck');
        var jAttemptCells = jRow.children('.attemptCell').find(':text').parent();
        var jExtraInputs = jRow.children('.additionalCell').children('.extra');
        var semesterRate = parseInt(jRow.children('.semesterRateResultCell').text());
        var autopass = jAutoPassCheckBox[0].checked;
        var absence = jAbsenceCheckBoxes[0].checked;
        if (semesterRate < 60 || absence) {
            jAutoPassCheckBox.attr('disabled', true);
        } else {
            jAutoPassCheckBox.removeAttr('disabled');
        }

        // суммарный добор
        var extraRate = jExtraInputs.aggregate((cur, elem) => +$(elem).val() + cur);
        var row = +jRow.attr('id').substr(4); // одинаковая для всех

        // Определим экзамен
        var curExamCell = 0;
        var curExamRate = 0;
        var curExamCellIndex = 0;
        var index = 0;
        var dependentCells = [[0, 0], [0, 0], [0, 0]]; // размер зависит от jAttemptCells.length
        jAttemptCells.each(function () {
            var needToRemeber = (curExamCell == 0); // инициализация первым экзаменом

            var id = $(this).attr('id');
            var col = +id.substr(4, 1);
            var neighborAbsenceCheck = $($(`#absence_${col}_${row}`).children()[0]);

            dependentCells[index][0] = $(this);
            dependentCells[index][1] = neighborAbsenceCheck.parent();

            var strVal = $(this).children()[0].value;
            var examRate = +strVal;
            if (strVal === '') {
                examRate = -1; // спец значение, означает 'не было оценки за экзамен'
                if (neighborAbsenceCheck.is(':checked')) {// неявка = 0 баллов
                    examRate = 0;
                    needToRemeber = true;
                }
            } else
                needToRemeber = true;

            if (needToRemeber) {
                curExamCell = $(this);
                curExamRate = examRate;
                curExamCellIndex = index;
            }

            index++;
        });

        var state = deriveStudentState(semesterRate, extraRate, curExamRate, autopass, curExamCellIndex == 0);

        // Выставляем доступность ячеек

        // Доступность добора
        var needAccess = !(state == StudentState.successful ||
        state == StudentState.beforeExam ||
        state == StudentState.notPassed);
        setCellAccess($(jExtraInputs[0]), needAccess);

        // Доступность экзаменов
        var examAccess = [false, false, false];
        if (state == StudentState.successful || state == StudentState.beforeExam)
            examAccess[0] = true; // только основная сдача
        else if (state == StudentState.admitted)
            examAccess[1] = true; // только пересдача 1
        else if (state == StudentState.admittedAndPassed)
            examAccess[curExamCellIndex] = true; // только текущая пересдача
        else if (state == StudentState.notPassed || state == StudentState.admittedAndNotPassed) {
            examAccess[curExamCellIndex] = true; // текущая сдача и следующий
            if (++curExamCellIndex < 3) // 3 == jAttemptCells.length
                examAccess[curExamCellIndex] = true;
        }

        for (index = 0; index < 3; ++index) { // 3 == jAttemptCells.length
            var examCell = dependentCells[index][0];
            var absenceCell = dependentCells[index][1];
            var absenceChecked = absenceCell.children()[0].checked;

            needAccess = examAccess[index] && !absenceChecked;
            setCellAccess(examCell.children()[0], needAccess);
            needAccess = examAccess[index]; //&& (val == 0);
            setCellAccess(absenceCell.children()[0], needAccess);
        }
    }

    function controlVisualization() {
        $('.studentsRate').children().children('tr')
            .each(function () {
                if (($(this).prop('id') !== '') && ($(this).attr('id').substr(0, 3) === 'row')) {
                    controlRowVisualization($(this));
                }
            });
    }

    // Скрываем все остальные группы
    // 0 - показать все
    function filterGroups(groupID) {
        if (groupID == 0) {
            $('.studentsRate tbody')
                .children()
                .each((_, elem) => $(elem).show());
        } else {
            $('.studentsRate tbody')
                .children(':gt(2)')
                .each(function () {
                    if ($(this).hasClass(`group_${groupID}`))
                        $(this).show();
                    else
                        $(this).hide();
                });
        }
    }

    var setCursorLocation = function (jThis) {
        g_col = parseInt(jThis.attr('id').substr(4));
        g_row = parseInt(jThis.parent('tr').attr('id').substr(4))
    };

    var setRowBackground = function (color) {
        let $cols = $(`td#col_${g_col}`);
        let $rows = $(`tr#row_${g_row}`);
        let $rowStatic = $rows.find('.staticCell');

        $cols.filter('.commonCell')
            .add($cols.filter('.staticCell'))
            .add($rows.find('.commonCell'))
            .add($rowStatic)
            .children('input')
            .add($rowStatic)
            .each((_, elem) => $(elem).css('background-color', color));
    };

    var defaultCellBackground = '#fff';
    var focusCellBackground = '#f1f1f1';

    // Ставим подстветку
    function TdFocus(jThis) {
        setCursorLocation(jThis);
        g_oldRateCell = jThis.children('input').val();

        setRowBackground(focusCellBackground);
        jThis.children('input').css('background-color', defaultCellBackground);
    }

    // Убираем подстветку
    function TdUnFocus() {
        setRowBackground(defaultCellBackground);
    }

    function TdInfo(jThis) {
        var disciplinePassRate = 60; // credit pass rate
        if (disciplineType === 'exam')
            disciplinePassRate = 38;

        // Получаем подмодуль
        var jCurSubmoduleInfo = $(`.RatingTableSubmodulesInfo .col_${g_col}:first`);
        var jCurSubmoduleHead = $(`.RatingTableSubmodulesHead .col_${g_col}:first`);

        g_submoduleID = parseInt(jCurSubmoduleInfo.attr('id'));
        g_submoduleTitle = jCurSubmoduleHead.text();
        if (jCurSubmoduleHead.length < 1 && $('.RatingTableModulesHead .bonus').length > 0)
            g_submoduleTitle = 'Бонусные баллы';
        g_submoduleMaxRate = parseInt($(`.RatingTableSubmodulesHeadMaxRate .col_${g_col}`).text());


        // Проверяем допустимое значение (только для добора)
        if (jThis.attr('class').indexOf('additionalCell') >= 0) {
            var semesterRate = parseInt(jThis.siblings('.semesterRateResultCell').text());
            if (semesterRate <= disciplinePassRate)
                g_submoduleMaxRate = disciplinePassRate - semesterRate;
            else
                g_submoduleMaxRate = 0;
        }

        // Получаем студента
        g_studentID = jThis.siblings('.studentCell').attr('id');
        g_studentID = g_studentID.substr(8);

        g_stdName = jThis.siblings('.studentCell').text();

        jTdInfo_wrap.show();
        jTdInfo.children('#student').children('b').html(g_stdName);
        jTdInfo.children('#submodule').children('b').html(g_submoduleTitle);
        jTdInfo.children('#maxRate').children('b').html(g_submoduleMaxRate);
    }

    function UnsetTdInfo(jThis) {
        jTdInfo_wrap.hide();
        g_submoduleID = null;
        g_studentID = null;
        g_submoduleMaxRate = 0;
    }

    var setRate = function(newRate, jThis, oldRate, rateResult, bonus) {
        $.postJSON(URLdir + 'handler/rating/setRate',
            {
                studentID: g_studentID, 
                submoduleID: g_submoduleID,
                rate: newRate
            }
        ).success(data => {
            if (data.success !== true) {
                jThis.children('input').val(oldRate);
                EventInspector.error('Не удалось добавить/изменить балл');
                return;
            }

            var correctRate = (rateResult + bonus > 100) ? '100+' : rateResult + bonus;
            jThis.siblings('.rateResultCell').text(correctRate);

            // Открываем доступ к след. ячейке добора баллов
            if (jThis.hasClass('additionalCell')) {
                var nextAdditionalCell = $('#col_' + (g_col + 1) + '_row_' + g_row);
                var placeholderMaxVal = (rateResult < 60) ? (60 - rateResult) : 0;//(60 - oldRate);

                if (nextAdditionalCell.hasClass('additionalCell')) {
                    var placeholderMax = (placeholderMaxVal > 0) ? placeholderMax = 'макс. ' + placeholderMaxVal : '---';
                    nextAdditionalCell.find('input').attr('placeholder', placeholderMax);
                }
            }
            EventInspector.success('Балл добавлен/изменен');
        }).fail(jqXHR => {
            jThis.children('input').val(oldRate);
            switch (jqXHR.status) {
                case 400:
                    EventInspector.error(optionsViewData[option].messageFail);
                    break;
                case 403:
                    EventInspector.error('Сессия истекла');
                    window.location.replace(URLdir);
                    break;
                default:
                    EventInspector.error(' ' + jqXHR.status);
            }
        });
    };

    function Rating(jThis, oldRate) {
        oldRate = parseInt(oldRate);

        // Здесь jThis - div rateCell, а не input, который является дочкой
        if (cancelFlag) {
            var str = '';
            if (oldRate != -1)
                str = oldRate;
            jThis.children('input').val(str);
            cancelFlag = false;
            return;
        }

        var newRate = -1; // если пустая строка в ячейке, значит ничего не поставлено
        if (jThis.children('input').val() !== '')
            newRate = parseInt(jThis.children('input').val());
        if (newRate == oldRate)
            return;

        // блокируем ячейку пока не обработаем коллбек
        jThis.children('input').attr('disabled', true);

        var rateResult = Math.max(0, newRate);
        var bonus = parseInt(jThis.siblings('.bonus').text());

        // считаем баллы по строке
        if (pageType === 'exam') //(jThis.attr('class').indexOf('attemptCell') >= 0)
        {
            // страница сессии
            rateResult += parseInt(jThis.siblings('.semesterRateResultCell').text());
            jThis.siblings('.additionalCell').aggregate((init, elem) => +$(elem).children('input').val() + init);
        }
        if (newRate === -1)
            rateResult += jThis.siblings('.attemptCell').not('.autoPass').not('.absenceCell')
                .aggregate((init, elem) => Math.max(init, +$(elem).find('input').val()));

        if (newRate <= g_submoduleMaxRate)
            setRate(newRate, jThis, oldRate, rateResult, bonus);
        else {
            var cellRate = (oldRate <= g_submoduleMaxRate)
                ? ((oldRate != -1) ? oldRate : '')
                : '0';
            jThis.children('input').val(cellRate);
            EventInspector.error('Текущий балл превышает максимальный для данного модуля');
        }
        jThis.children('input').removeAttr('disabled');
    }

    jCommonCell.mouseenter(function () {
        if (g_isFocusCell === false)
            TdFocus($(this));
    });

    jCommonCell.mouseleave(function () {
        if (g_isFocusCell === false)
            TdUnFocus();
    });

    var oldRate = 0;
    jCommonCell.focusin(function () {
        g_isFocusCell = true;
        TdFocus($(this));
        TdInfo($(this));

        var value = $(this).children('input').val();
        oldRate = (value !== '') ? +value : -1;
    });

    jCommonCell.focusout(function () {
        g_isFocusCell = false;
        Rating($(this), oldRate);

        TdUnFocus();
        UnsetTdInfo($(this));
        controlRowVisualization($(this).parent());
    });

    var Direction = {
        Up: 0,
        Right: 1,
        Down: 2,
        Left: 3
    };

    /**
     * @param direction Direction в каком направлении искать следующую ячейку для перемещения фокуса
     * @return jQuery клетка, в которую надо переместиться или null, если в этом направлении нет подходящих ячеек
     */
    function getDesiredCell(direction) {
        /** Будем искать ячейку в направлении {@see direction} до тех пока не найдем её и пока не
         *  кончатся ячейки в этом направлении */
        var row = g_row;
        var col = g_col;
        var currentCell;
        switch (direction) {
            case Direction.Up:
                do {
                    row--;
                    currentCell = $(`tr#row_${row} td#col_${g_col}.commonCell`)
                } while ((currentCell.length > 0) &&
                (currentCell.children('input').attr('disabled') == 'disabled'));
                return currentCell;
            case Direction.Right:
                do {
                    col++;
                    currentCell = $(`tr#row_${g_row} td#col_${col}.commonCell`)
                } while ((currentCell.length > 0) &&
                (currentCell.children('input').attr('disabled') == 'disabled'));
                return currentCell;
            case Direction.Down:
                do {
                    row++;
                    currentCell = $(`tr#row_${row} td#col_${g_col}.commonCell`)
                } while ((currentCell.length > 0) &&
                (currentCell.children('input').attr('disabled') == 'disabled'));
                return currentCell;
            case Direction.Left:
                do {
                    col--;
                    currentCell = $(`tr#row_${g_row} td#col_${col}.commonCell`)
                } while ((currentCell.length > 0) &&
                (currentCell.children('input').attr('disabled') == 'disabled'));
                return currentCell;
        }
        return null;
    }

    jCommonCell.keydown(function (e) {
        var row = g_row;
        var col = g_col;
        var direction;
        switch (e.keyCode) {
            case 27:
                cancelFlag = true; // esc
            case 13: // enter
            case 40: // down arrow
                direction = Direction.Down;
                break;
            case 38: // up arrow
                direction = Direction.Up;
                break;
            case 39: // right arrow
                direction = Direction.Right;
                break;
            case 37: // left arrow
                direction = Direction.Left;
                break;
            default:
                return;
        }
        var whereToMoveFocus = getDesiredCell(direction);
        if (whereToMoveFocus.length > 0) {
            TdUnFocus();
            g_row = row;
            g_col = col;
            whereToMoveFocus.children('input').focus();
        } else
            $(this).children('input').blur();
    });

    $('.commonCell input').focusin(function () {
        $(this).select();
    });

    // При нажатии на элемент commonCell дочерный input получает фокус
    jCommonCell.click(function () {
        $(this).children('input').focus();
    });

    // В inputCredit (где баллы вводить) разрешаем вводить только цифры
    jCommonCell.children('input').keydown(function (event) {
        KeyDownOnlyNumber(event);
    });

    // Нажатие на чекбокс "Неявка"
    $('.absenceCheck').click(function (event) {
        // prepare
        var option = 'absence';
        var checked = $(this)[0].checked;
        if (!checked)
            option += '_unset';
        var thisTd = $(this).parent();
        g_col = thisTd.attr('id').substr(8, 1); // absence_C
        g_row = thisTd.attr('id').substr(10, 1); // absence_C_R

        // call
        TdInfo(thisTd);
        var errCode = setExamPeriodOption($(this), option);
        UnsetTdInfo(thisTd);

        // test
        if (errCode > 0)
            $(this).prop('checked', !checked); // invert check;
        controlRowVisualization(thisTd.parent());
    });

    $('.autoPassCheck').click(function (event) {
        // prepare
        var option = 'pass';
        var checked = $(this)[0].checked;
        if (!checked)
            option += '_unset';
        var thisTd = $(this).parent();
        g_col = thisTd.attr('id').substr(9, 1); // autopass_C
        g_row = thisTd.attr('id').substr(11, 1); // autopass_C_R

        // call
        TdInfo(thisTd);
        var errCode = setExamPeriodOption($(this), option);
        UnsetTdInfo(thisTd);

        // test
        if (errCode > 0)
            $(this).prop('checked', !checked); // invert check;
        controlRowVisualization(thisTd.parent());
    });


    function setExamPeriodOption(jThis, option) {
        jThis.attr('disabled', true);

        $.postJSON(URLdir + 'handler/rating/SetExamPeriodOption',
            {
                studentID: g_studentID,
                submoduleID: g_submoduleID,
                option: '' + optionsViewData[option].sendingOption // cast null to "null"
            })
        .done(data => EventInspector.success(optionsViewData[option].messageOK))
        .fail(jqXHR => {
            jThis.prop("checked", !jThis.prop("checked"));
            switch (jqXHR.status) {
                case 400:
                    EventInspector.error(optionsViewData[option].messageFail);
                    break;
                case 403:
                    EventInspector.error('Сессия истекла');
                    window.location.replace(URLdir);
                    break;
                default:
                    EventInspector.error(' ' + jqXHR.status);
            }
        }).always(() => jThis.removeAttr('disabled'));
    }

    // Фильтр по группе
    $('.groupSelector').change(function () {
        var group = $(this).val();
        if (!(group >= 0))
            return;

        filterGroups(group);
        $.post(
            URLdir + 'handler/rating/SelectGroup',
            {
                'disciplineID': g_disciplineID,
                'groupSelected': group
            }
        );
    });

    // Скачать таблицу оценивания в формате excel // depricated
    $('.downloadExcel').click(function () {
        $.fileDownload(URLdir + 'handler/FileCreator/GenerateExcelRatingTable', {
            httpMethod: 'POST',
            data: {'disciplineID': g_disciplineID},
            successCallback: function () {},
            failCallback: function () {}
        });
    });

    // Ведомость в формате excel
    $('body').on('click', '.downloadExcelStatement', function () {

        var groupID = parseInt($(this).attr('id').substr(6));
        var jStageSelector = $('#stageSelector_' + groupID);

        $.fileDownload(URLdir + 'handler/FileCreator/GenerateFinalForm', {
            httpMethod: 'POST',
            data: {
                'disciplineID': g_disciplineID,
                'groupID': groupID,
                'stage': parseInt(jStageSelector.val())
            },
            successCallback: function () {},
            failCallback: function () {}
        });
    });
});
